class OriginalPicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :license
  has_one :resource_picture
  
  before_destroy :destroy_with_file
  
  def validate
    errors.add(:filesize, 'size over(1MB)') if self.filesize > 1000000
  end
  
  def supply_default art
    self.license_id = art.default_license_id if self.license_id.blank?
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list artist_id, opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => ['artist_id = ?', artist_id], 
      :order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size}
    )
    OriginalPicture.find(:all, opt)
  end
  
  def self.list_opt
    {:include => [:artist, :license, :resource_picture]}
  end
  
  def self.list_json_opt
    {:include => [:resource_picture, :artist, :license]}
  end
  
  def self.show cid, author, opt = {}
    pic = OriginalPicture.find(cid, :include => self.show_include_opt(opt))
    raise ActiveRecord::Forbidden unless pic.own?(author)
    pic
  end
  
  def self.show_include_opt opt = {}
    res = [:license]
    res.push(opt[:include]) if opt[:include]
    res
  end
  
  def self.show_json_include_opt
    {:include => :license}
  end
  
  def destroy_with_file
    PictureIO.original_picture_io.delete self.filename
    self.resource_picture.destroy
  end
  
  def dext
    self.ext.downcase
  end
  
  def filename
    "#{self.id}.#{self.dext}"
  end
  
  def mime_type
    "image/#{self.dext}"
  end
  
  def url
    '/original_pictures/' + filename
  end
  
  def store(picture_data, art, lid = nil)
    res = false
    begin
      mgk = Magick::Image.from_blob(picture_data).shift
    rescue 
      self.errors.add :base, 'magick failed'
      return false
    end
    self.attributes = {:ext => mgk.format.downcase, :width => mgk.columns, :height => mgk.rows, 
      :filesize => mgk.filesize, :artist_id => art.id, 
      :license_id => lid.blank? ? art.default_license_id : lid.to_i
    }
    OriginalPicture.transaction do
      if res = self.save
        if res = PictureIO.original_picture_io.put(picture_data, self.filename)
          rp = ResourcePicture.update_picture(self)
          unless rp.store(mgk)
            PictureIO.original_picture_io.delete(self.filename)
            self.errors.add :base, 'resource picture copying error'
            raise ActiveRecord::Rollback
          end
        else
          self.errors.add :base, 'original picture io does not work'
          raise ActiveRecord::Rollback
        end
      end
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.original_picture_io.get self.filename, subdir
  end
  
  def own? author
    return false unless author
    return false unless author.artist?
    self.artist_id == author.artist.id
  end
  
end
